@@ -1,10 +1,10 @@
 import locust
-import core
 import runners
 
 import gevent
 import sys
 import os
+import signal
 import inspect
 import time
 import logging
@@ -12,7 +12,7 @@ from optparse import OptionParser
 
 import web
 from log import setup_logging, console_logger
-from stats import stats_printer, RequestStats, print_percentile_stats, print_error_report, print_stats
+from stats import stats_printer, print_percentile_stats, print_error_report, print_stats
 from inspectlocust import print_task_ratio, get_task_ratio_dict
 from core import Locust, WebLocust
 from runners import MasterLocustRunner, SlaveLocustRunner, LocalLocustRunner
@@ -141,15 +141,6 @@ def parse_options():
         help="Enables the auto tuning ramping feature for finding highest stable client count. NOTE having ramp enabled will add some more overhead for additional stats gathering"
     )
     
-    # if the HTTP client should set gzip headers and try to use gzip decoding
-    parser.add_option(
-        '--gzip',
-        action='store_true',
-        dest='gzip',
-        default=False,
-        help="If present, the HTTP client will set request header Accept-Encoding: gzip, and try to gzip decode the response data."
-    )
-    
     # if we shgould print stats in the console
     parser.add_option(
         '--print-stats',
@@ -298,11 +289,7 @@ def load_locustfile(path):
 
 def main():
     parser, options, arguments = parse_options()
-    #print "Options:", options, dir(options)
-    #print "Arguments:", arguments
-    #print "largs:", parser.largs
-    #print "rargs:", parser.rargs
-    
+
     # setup logging
     setup_logging(options.loglevel, options.logfile)
     logger = logging.getLogger(__name__)
@@ -367,9 +354,6 @@ def main():
         logger.info("Starting web monitor on port 8089")
         main_greenlet = gevent.spawn(web.start, locust_classes, options.hatch_rate, options.num_clients, options.num_requests, options.ramp)
     
-    # enable/disable gzip in WebLocust's HTTP client
-    WebLocust.gzip = options.gzip
-
     if not options.master and not options.slave:
         runners.locust_runner = LocalLocustRunner(locust_classes, options.hatch_rate, options.num_clients, options.num_requests, options.host)
         # spawn client spawning/hatching greenlet
@@ -386,18 +370,28 @@ def main():
         # spawn stats printing greenlet
         gevent.spawn(stats_printer)
     
+    def shutdown(code=0):
+        """
+        Shut down locust by firing quitting event, printing stats and exiting
+        """
+        logger.info("Shutting down, bye..")
+        events.quitting.fire()
+        print_stats(runners.locust_runner.request_stats)
+        print_percentile_stats(runners.locust_runner.request_stats)
+        print_error_report()
+        sys.exit(code)
+    
+    # install SIGTERM handler
+    def sig_term_handler():
+        logger.info("Got SIGTERM signal")
+        shutdown(0)
+    gevent.signal(signal.SIGTERM, sig_term_handler)
+    
     try:
         logger.info("Starting Locust %s" % version)
         main_greenlet.join()
     except KeyboardInterrupt, e:
-        events.quitting.fire()
-        time.sleep(0.2)
-        print_stats(runners.locust_runner.request_stats)
-        print_percentile_stats(runners.locust_runner.request_stats)
-        print_error_report()
-        logger.info("Got KeyboardInterrupt. Exiting, bye..")
-
-    sys.exit(0)
+        shutdown(0)
 
 if __name__ == '__main__':
     main()
