@@ -1,4 +1,6 @@
+# coding=UTF-8
 import socket
+import traceback
 import warnings
 import random
 import logging
@@ -34,6 +36,7 @@ class LocustRunner(object):
         self.locusts = Group()
         self.state = STATE_INIT
         self.hatching_greenlet = None
+        self.exceptions = {}
         
         # register listener that resets stats when hatching is complete
         def on_hatch_complete(count):
@@ -60,10 +63,10 @@ class LocustRunner(object):
         returns a list "bucket" with the weighted locusts
         """
         bucket = []
-        weight_sum = sum((locust.weight for locust in self.locust_classes if locust.tasks))
+        weight_sum = sum((locust.weight for locust in self.locust_classes if locust.task_set))
         for locust in self.locust_classes:
-            if not locust.tasks:
-                warnings.warn("Notice: Found locust (%s) got no tasks. Skipping..." % locust.__name__)
+            if not locust.task_set:
+                warnings.warn("Notice: Found Locust class (%s) got no task_set. Skipping..." % locust.__name__)
                 continue
 
             if self.host is not None:
@@ -107,9 +110,7 @@ class LocustRunner(object):
                 occurence_count[locust.__name__] += 1
                 def start_locust(_):
                     try:
-                        locust()()
-                    except RescheduleTaskImmediately:
-                        pass
+                        locust().run()
                     except GreenletExit:
                         pass
                 new_locust = self.locusts.spawn(start_locust, locust)
@@ -147,6 +148,8 @@ class LocustRunner(object):
         if self.state != STATE_RUNNING and self.state != STATE_HATCHING:
             RequestStats.clear_all()
             RequestStats.global_start_time = time()
+            self.exceptions = {}
+
         # Dynamically changing the locust count
         if self.state != STATE_INIT and self.state != STATE_STOPPED:
             self.state = STATE_HATCHING
@@ -175,7 +178,23 @@ class LocustRunner(object):
         self.locusts.kill(block=True)
         self.state = STATE_STOPPED
 
+    def log_exception(self, node_id, msg, formatted_tb):
+        key = hash(formatted_tb)
+        row = self.exceptions.setdefault(key, {"count": 0, "msg": msg, "traceback": formatted_tb, "nodes": set()})
+        row["count"] += 1
+        row["nodes"].add(node_id)
+        self.exceptions[key] = row
+
 class LocalLocustRunner(LocustRunner):
+    def __init__(self, locust_classes, hatch_rate, num_clients, num_requests, host=None):
+        super(LocalLocustRunner, self).__init__(locust_classes, hatch_rate, num_clients, num_requests, host)
+
+        # register listener thats logs the exception for the local runner
+        def on_locust_error(locust, e, tb):
+            formatted_tb = "".join(traceback.format_tb(tb))
+            self.log_exception("local", str(e), formatted_tb)
+        events.locust_error += on_locust_error
+
     def start_hatching(self, locust_count=None, hatch_rate=None, wait=False):
         self.hatching_greenlet = gevent.spawn(lambda: super(LocalLocustRunner, self).start_hatching(locust_count, hatch_rate, wait=wait))
         self.greenlet = self.hatching_greenlet
@@ -216,7 +235,7 @@ class MasterLocustRunner(DistributedLocustRunner):
         self.client_stats = {}
         self.client_errors = {}
         self._request_stats = {}
-        
+
         self.server = rpc.Server()
         self.greenlet = Group()
         self.greenlet.spawn(self.client_listener).link_exception()
@@ -225,6 +244,11 @@ class MasterLocustRunner(DistributedLocustRunner):
         def on_slave_report(client_id, data):
             self.clients[client_id].user_count = data["user_count"]
         events.slave_report += on_slave_report
+        
+        # register listener that sends quit message to slave nodes
+        def on_quitting():
+            self.quit()
+        events.quitting += on_quitting
     
     @property
     def user_count(self):
@@ -242,6 +266,7 @@ class MasterLocustRunner(DistributedLocustRunner):
         
         if self.state != STATE_RUNNING and self.state != STATE_HATCHING:
             RequestStats.clear_all()
+            self.exceptions = {}
         
         for client in self.clients.itervalues():
             data = {"hatch_rate":slave_hatch_rate, "num_clients":slave_num_clients, "num_requests": self.num_requests, "host":self.host, "stop_timeout":None}
@@ -254,6 +279,11 @@ class MasterLocustRunner(DistributedLocustRunner):
         for client in self.clients.hatching + self.clients.running:
             self.server.send(Message("stop", None, None))
     
+    def quit(self):
+        for client in self.clients.itervalues():
+            self.server.send(Message("quit", None, None))
+        self.greenlet.kill(block=True)
+    
     def client_listener(self):
         while True:
             msg = self.server.recv()
@@ -261,6 +291,9 @@ class MasterLocustRunner(DistributedLocustRunner):
                 id = msg.node_id
                 self.clients[id] = SlaveNode(id)
                 logger.info("Client %r reported as ready. Currently %i clients ready to swarm." % (id, len(self.clients.ready)))
+                ## emit a warning if the slave's clock seem to be out of sync with our clock
+                #if abs(time() - msg.data["time"]) > 5.0:
+                #    warnings.warn("The slave node's clock seem to be out of sync. For the statistics to be correct the different locust servers need to have synchronized clocks.")
             elif msg.type == "client_stopped":
                 del self.clients[msg.node_id]
                 if len(self.clients.hatching + self.clients.running) == 0:
@@ -280,6 +313,8 @@ class MasterLocustRunner(DistributedLocustRunner):
                 if msg.node_id in self.clients:
                     del self.clients[msg.node_id]
                     logger.info("Client %r quit. Currently %i clients connected." % (msg.node_id, len(self.clients.ready)))
+            elif msg.type == "exception":
+                self.log_exception(msg.node_id, msg.data["msg"], msg.data["traceback"])
 
     @property
     def slave_count(self):
@@ -310,7 +345,13 @@ class SlaveLocustRunner(DistributedLocustRunner):
         def on_quitting():
             self.client.send(Message("quit", None, self.client_id))
         events.quitting += on_quitting
-    
+
+        # register listener thats sends locust exceptions to master
+        def on_locust_error(locust, e, tb):
+            formatted_tb = "".join(traceback.format_tb(tb))
+            self.client.send(Message("exception", {"msg" : str(e), "traceback" : formatted_tb}, self.client_id))
+        events.locust_error += on_locust_error
+
     def worker(self):
         while True:
             msg = self.client.recv()
@@ -326,7 +367,10 @@ class SlaveLocustRunner(DistributedLocustRunner):
                 self.stop()
                 self.client.send(Message("client_stopped", None, self.client_id))
                 self.client.send(Message("client_ready", None, self.client_id))
-
+            elif msg.type == "quit":
+                logger.info("Got quit message from master, shutting down...")
+                self.stop()
+                self.greenlet.kill(block=True)
 
     def stats_reporter(self):
         while True:
